MODULE IRQFIQ;
(* =========================================================================  
   Example ARM Oberon-07 Module  
 
   Description:
     Demo showing an IRQ handler interrupted by an FIQ handler
  
   Target: 
     LPC21xx systems
   
   References: 
     NXP LPC2xxx User Manuals UM10114, UM10139, UM10161 & UM10275
     NXP Application Note AN10254: 
       "Handling Interrupts Using FIQ and IRQ for LPC2000 Microcontrollers" 
     
   
   (c) 2011 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

IMPORT LPC, Main, Out, SYSTEM;

VAR
  inIRQ: BOOLEAN;
  countIRQ, countFIQ, lastCount: INTEGER;
 
PROCEDURE FIQTimerHandler[4]; (* 4 for IRQ or FIQ *)
BEGIN
  inIRQ := FALSE;
  INC(countFIQ);
  (* Clear the Timer1 MR0 interrupt *)
  SYSTEM.PUT(LPC.T1IR, {0})
END FIQTimerHandler;


PROCEDURE IRQTimerHandler[4]; (* 4 for IRQ or FIQ *)
BEGIN
  inIRQ := TRUE;
  INC(countIRQ);
  (* Clear the Timer0 MR0 interrupt *)
  SYSTEM.PUT(LPC.T0IR, {0});                       
  WHILE inIRQ DO END;
  (* Update the VIC priority hardware *)  
  SYSTEM.PUT(LPC.VICVectAddr, 0)
END IRQTimerHandler;


PROCEDURE InitIRQ*();
CONST
  (* Timer0 interrupt ID and associated bit *)
  timer0 = 4;  
  timer0Bit = {timer0};
VAR
  select: SET;
BEGIN
  (* Configure Timer0 *)
  (* Clear the pre-scale register *)
  SYSTEM.PUT(LPC.T0PR, 0);
  (* Timer match value *)
  SYSTEM.PUT(LPC.T0MR0, 100000000);                     
  (* MR0 Interrupt (MR0I: {0}) and Reset (MR0R: {1}) on match *)  
  SYSTEM.PUT(LPC.T0MCR, {0, 1});   
     
  (* Configure the Vectored Interrupt Controller *) 
  (* Use IRQ not FIQ for timer0 *)  
  SYSTEM.GET(LPC.VICIntSelect, select);
  SYSTEM.PUT(LPC.VICIntSelect, select - timer0Bit);   
  (* Install handler *)
  SYSTEM.PUT(LPC.VICVectAddr0, SYSTEM.ADR(IRQTimerHandler)); 
  (* Assign timer0 to slot 0 and set enable bit *)  
  SYSTEM.PUT(LPC.VICVectCntl0, timer0 + ORD({5}));   
  (* Enable Timer0 interrupt *)
  SYSTEM.PUT(LPC.VICIntEnable, timer0Bit); 
    
  (* Start the timer *)  
  SYSTEM.PUT(LPC.T0TCR, {0})                           

END InitIRQ;


PROCEDURE InitFIQ*();
CONST
  (* Timer1 interrupt ID and associated bit *)
  timer1 = 5;  
  timer1Bit = {timer1};
  (* Interrupt handler addresses are in RAM *)
  FIQHandlerAddress = 04000003CH;
VAR
  select: SET;
BEGIN
  (* Configure Timer1 *)
  (* Clear the pre-scale register *)
  SYSTEM.PUT(LPC.T1PR, 0);
  (* Timer match value *)
  SYSTEM.PUT(LPC.T1MR0, 10000000);                     
  (* MR0 Interrupt (MR0I: {0}) and Reset (MR0R: {1}) on match *)  
  SYSTEM.PUT(LPC.T1MCR, {0, 1});   

  (* Configure the Vectored Interrupt Controller *) 
  (* to use FIQ not IRQ for Timer *)  
  SYSTEM.GET(LPC.VICIntSelect, select);
  SYSTEM.PUT(LPC.VICIntSelect, select + timer1Bit); 
   
  (* Install FIQ handler *)
  SYSTEM.PUT(FIQHandlerAddress, SYSTEM.ADR(FIQTimerHandler));
  
  (* Enable Timer1 interrupt *)
  SYSTEM.PUT(LPC.VICIntEnable, timer1Bit); 
    
  (* Start the timer *)  
  SYSTEM.PUT(LPC.T1TCR, {0})                           

END InitFIQ;


PROCEDURE Run();
BEGIN
  countIRQ := 0;
  countFIQ := 0;
  lastCount := 0;
  InitIRQ();
  InitFIQ();
  WHILE TRUE DO
    IF countFIQ > lastCount THEN 
      Out.String("FIQs: "); Out.Int(countFIQ, 0);
      Out.String(", IRQs: "); Out.Int(countIRQ, 0);
      Out.Ln;
      lastCount := countFIQ
    END 
  END
END Run;

BEGIN
  Out.String("IRQFIQ Started");
  Out.Ln;
  Run()
END IRQFIQ.


